@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2007-2019 CS-SI. All Rights Reserved.
+* Copyright (C) 2007-2020 CS GROUP - France. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -160,20 +160,18 @@ static bufpool_t *evict_from_memory(void)
         size_t prev_len = 0;
         bufpool_t *bp = NULL, *evict = NULL;
 
-        while ( 1 ) {
-                gl_lock_lock(destroy_prevention);
+        gl_lock_lock(destroy_prevention);
 
+        while ( 1 ) {
                 gl_lock_lock(mutex);
                 bp = prelude_list_get_next(&pool_list, bp, bufpool_t, list);
                 gl_lock_unlock(mutex);
 
                 if ( ! bp ) {
-                        gl_lock_unlock(destroy_prevention);
                         break;
                 }
 
                 gl_lock_lock(bp->mutex);
-                gl_lock_unlock(destroy_prevention);
 
                 if ( bp->failover ) {
                         gl_lock_unlock(bp->mutex);
@@ -192,14 +190,14 @@ static bufpool_t *evict_from_memory(void)
                         gl_lock_unlock(bp->mutex);
         }
 
-
         if ( evict ) {
                 flush_bufpool_to_disk(evict);
                 gl_lock_unlock(evict->mutex);
-                return evict;
         }
 
-        return NULL;
+        gl_lock_unlock(destroy_prevention);
+
+        return evict;
 }
 
 
@@ -308,6 +306,8 @@ int bufpool_get_message(bufpool_t *bp, prelude_msg_t **out)
 
 int bufpool_new(bufpool_t **bp, const char *filename)
 {
+        int ret;
+
         *bp = malloc(sizeof(**bp));
         if ( ! *bp )
                 return -1;
@@ -323,10 +323,24 @@ int bufpool_new(bufpool_t **bp, const char *filename)
                 return prelude_error_from_errno(errno);
         }
 
+        ret = prelude_failover_new(&(*bp)->failover, (*bp)->filename);
+        if ( ret < 0 ) {
+                free((*bp)->filename);
+                free(*bp);
+                return ret;
+        }
+
+        (*bp)->count = prelude_failover_get_available_msg_count((*bp)->failover);
+        if ( (*bp)->count == 0 ) {
+                prelude_failover_destroy((*bp)->failover);
+                (*bp)->failover = NULL;
+        }
+
         gl_lock_init((*bp)->mutex);
 
         gl_lock_lock(mutex);
         prelude_list_add_tail(&pool_list, &(*bp)->list);
+        disk_msgcount += (*bp)->count;
         gl_lock_unlock(mutex);
 
         return 0;
@@ -340,7 +354,14 @@ void bufpool_destroy(bufpool_t *bp)
         gl_lock_unlock(destroy_prevention);
 
         gl_lock_lock(mutex);
+
         prelude_list_del(&bp->list);
+
+        if ( bp->failover )
+                disk_msgcount -= bp->count;
+        else
+                mem_msgcount -= bp->count;
+
         gl_lock_unlock(mutex);
 
         if ( bp->failover )
