@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2020 CS GROUP - France. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -31,7 +31,7 @@ typedef struct reverse_relay_receiver reverse_relay_receiver_t;
 
 void reverse_relay_set_receiver_dead(reverse_relay_receiver_t *rrr);
 
-int reverse_relay_set_receiver_alive(reverse_relay_receiver_t *rrr);
+int reverse_relay_set_receiver_alive(reverse_relay_receiver_t *rrr, server_generic_client_t *client);
 
 int reverse_relay_new_receiver(reverse_relay_receiver_t **rrr, server_generic_client_t *client, uint64_t analyzerid);
 
@@ -45,4 +45,6 @@ int reverse_relay_create_initiator(const char *arg);
 
 int reverse_relay_init(void);
 
+int reverse_relay_write_possible(reverse_relay_receiver_t *rrr, server_generic_client_t *client);
+
 #endif
