@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2020 CS GROUP - France. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <assert.h>
 
 #include <libprelude/prelude.h>
 #include <libprelude/prelude-log.h>
@@ -36,6 +37,7 @@
 
 #include "glthread/lock.h"
 
+#include "bufpool.h"
 #include "reverse-relaying.h"
 #include "server-generic.h"
 #include "sensor-server.h"
@@ -48,14 +50,12 @@
 struct reverse_relay_receiver {
         prelude_list_t list;
         uint64_t analyzerid;
-        prelude_failover_t *failover;
+        bufpool_t *failover;
 };
 
 
 
 static prelude_msgbuf_t *msgbuf;
-static PRELUDE_LIST(mqueue_list);
-static gl_lock_t mqueue_mutex = gl_lock_initializer;
 
 static prelude_bool_t no_receiver = TRUE;
 static prelude_list_t receiver_list[1024];
@@ -63,6 +63,8 @@ static gl_lock_t receiver_list_mutex = gl_lock_initializer;
 
 extern manager_config_t config;
 extern prelude_client_t *manager_client;
+static prelude_connection_pool_t *initiator = NULL;
+
 
 
 
@@ -72,6 +74,31 @@ static unsigned int get_list_key(uint64_t analyzerid)
 }
 
 
+static int connection_event_cb(prelude_connection_pool_t *pool,
+                               prelude_connection_pool_event_t event, prelude_connection_t *cnx)
+{
+        int ret;
+        server_generic_client_t *client;
+
+        if ( ! (event & PRELUDE_CONNECTION_POOL_EVENT_ALIVE) )
+                return 0;
+
+#if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
+        ret = fcntl(prelude_io_get_fd(prelude_connection_get_fd(cnx)), F_SETFL, O_NONBLOCK);
+        if ( ret < 0 )
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not set non blocking mode for client: %s", strerror(errno));
+#endif
+
+        ret = sensor_server_add_client(config.server[0], &client, cnx);
+        if ( ret < 0 )
+                prelude_log(PRELUDE_LOG_WARN, "error adding new client to reverse relay list.\n");
+
+        prelude_connection_set_data(cnx, client);
+
+        return 0;
+}
+
+
 
 static reverse_relay_receiver_t *get_next_receiver(prelude_list_t *receiver_list, prelude_list_t **iter)
 {
@@ -93,53 +120,100 @@ static reverse_relay_receiver_t *get_next_receiver(prelude_list_t *receiver_list
 
 
 
-static int write_all_client(reverse_relay_receiver_t *rrr, prelude_msg_t *msg, unsigned long count)
+static sensor_fd_t *find_client(reverse_relay_receiver_t *rrr)
 {
-        int i = 0, ret;
         sensor_fd_t *client;
-        prelude_list_t *tmp, *bkp;
+        prelude_list_t *tmp;
 
-        prelude_list_for_each_safe(sensor_server_get_list(rrr->analyzerid), tmp, bkp) {
+        prelude_list_for_each(sensor_server_get_list(rrr->analyzerid), tmp) {
                 client = prelude_list_entry(tmp, sensor_fd_t, list);
-
-                if ( client->ident != rrr->analyzerid )
-                        continue;
-
-                i++;
-                if ( count )
-                        server_generic_log_client((server_generic_client_t *) client, PRELUDE_LOG_INFO,
-                                                  "flushing %lu messages received while analyzer was offline.\n", count);
-
-                ret = sensor_server_write_client((server_generic_client_t *) client, prelude_msg_ref(msg));
-                if ( ret < 0 && prelude_error_get_code(ret) != PRELUDE_ERROR_EAGAIN )
-                         server_generic_client_close((server_generic_client_t *) client);
+                if ( client->ident == rrr->analyzerid )
+                        return client;
         }
 
-        return i;
+        return NULL;
 }
 
 
 
-int reverse_relay_set_receiver_alive(reverse_relay_receiver_t *rrr)
+static int write_client(sensor_fd_t *client, prelude_msg_t *msg)
+{
+        int ret;
+
+        ret = sensor_server_write_client((server_generic_client_t *) client, msg);
+        if ( ret < 0 ) {
+                if ( prelude_error_get_code(ret) != PRELUDE_ERROR_EAGAIN ) {
+                        prelude_perror(ret, "write client close cnx");
+                        server_generic_client_close((server_generic_client_t *) client);
+
+                } else {
+                        assert(client->wmsg);
+                        server_generic_notify_write_enable((server_generic_client_t *) client);
+                }
+        }
+
+        return ret;
+}
+
+
+
+int reverse_relay_write_possible(reverse_relay_receiver_t *rrr, server_generic_client_t *client)
 {
-        ssize_t size;
         prelude_msg_t *msg;
-        prelude_failover_t *failover = rrr->failover;
-        unsigned long avail = prelude_failover_get_available_msg_count(failover);
+        int ret, limit = 1024;
+        sensor_fd_t *sclient = (sensor_fd_t *) client;
 
-        while ( (size = prelude_failover_get_saved_msg(failover, &msg)) > 0 ) {
-                write_all_client(rrr, msg, avail);
-                prelude_msg_destroy(msg);
+        while ( --limit > 0 && (ret = bufpool_get_message(rrr->failover, &msg)) > 0 ) {
+                ret = write_client(sclient, msg);
+                if ( ret < 0 ) {
+                        if ( prelude_error_get_code(ret) == PRELUDE_ERROR_EAGAIN ) {
+                                ret = 0;
+                                break;
+                        }
 
-                avail = 0;
+                        return ret;
+                }
         }
 
-        if ( size < 0 ) {
-                prelude_perror((prelude_error_t) size, "could not retrieve saved message from disk");
-                return -1;
-        }
+        if ( ! sclient->wmsg && bufpool_get_message_count(rrr->failover) == 0 )
+                server_generic_notify_write_disable(client);
 
-        return 0;
+        else if ( ! limit )
+                server_generic_notify_write_enable(client);
+
+        return ret;
+}
+
+
+
+int reverse_relay_set_receiver_alive(reverse_relay_receiver_t *rrr, server_generic_client_t *client)
+{
+        ssize_t count = bufpool_get_message_count(rrr->failover);
+
+        if ( count )
+                server_generic_log_client((server_generic_client_t *) client, PRELUDE_LOG_INFO,
+                                          "flushing %lu messages received while analyzer was offline.\n", count);
+
+        /*
+         * It is possible that reverse_relay_send_prepared() has been called before we enter this callback.
+         * In this case, wmsg might be set.
+         */
+        if ( ((sensor_fd_t *) client)->wmsg )
+                return 0;
+
+        return reverse_relay_write_possible(rrr, client);
+}
+
+
+
+int reverse_relay_set_initiator_dead(prelude_connection_t *cnx)
+{
+        int ret = -1;
+
+        if ( initiator )
+                ret = prelude_connection_pool_set_connection_dead(initiator, cnx);
+
+        return ret;
 }
 
 
@@ -163,7 +237,7 @@ static reverse_relay_receiver_t *reverse_relay_search_receiver(uint64_t analyzer
 int reverse_relay_new_receiver(reverse_relay_receiver_t **rrr, server_generic_client_t *client, uint64_t analyzerid)
 {
         int ret;
-        char fname[PATH_MAX];
+        char bdir[PATH_MAX], fname[PATH_MAX];
         reverse_relay_receiver_t *new;
 
         new = reverse_relay_search_receiver(analyzerid);
@@ -174,38 +248,53 @@ int reverse_relay_new_receiver(reverse_relay_receiver_t **rrr, server_generic_cl
         if ( ! new )
                 return -1;
 
-        new->analyzerid = analyzerid;
-        prelude_client_profile_get_backup_dirname(prelude_client_get_profile(manager_client), fname, sizeof(fname));
-        snprintf(fname + strlen(fname), sizeof(fname) - strlen(fname), "/%" PRELUDE_PRIu64, analyzerid);
+        prelude_client_profile_get_backup_dirname(prelude_client_get_profile(manager_client), bdir, sizeof(bdir));
+        snprintf(fname, sizeof(fname), "%s/send-failover.%" PRELUDE_PRIu64, bdir, analyzerid);
 
-        ret = prelude_failover_new(&new->failover, fname);
+        ret = bufpool_new(&new->failover, fname);
         if ( ret < 0 ) {
-                prelude_perror(ret, "could not create failover");
-                free(new);
+                prelude_perror(ret, "error creating reverse relay failover");
                 return -1;
         }
 
+        new->analyzerid = analyzerid;
+
         gl_lock_lock(receiver_list_mutex);
         no_receiver = FALSE;
         prelude_list_add_tail(&receiver_list[get_list_key(analyzerid)], &new->list);
         gl_lock_unlock(receiver_list_mutex);
-        *rrr = new;
 
     out:
-        return reverse_relay_set_receiver_alive(new);
+        *rrr = new;
+        return reverse_relay_set_receiver_alive(new, client);
 }
 
 
 
 static int send_msgbuf(prelude_msgbuf_t *msgbuf, prelude_msg_t *msg)
 {
-        prelude_msg_set_data(msg, prelude_msgbuf_get_data(msgbuf));
+        int i, ret;
+        prelude_list_t *tmp;
+        prelude_msg_t *clone;
+        reverse_relay_receiver_t *receiver, *sender = prelude_msgbuf_get_data(msgbuf);
 
-        gl_lock_lock(mqueue_mutex);
-        prelude_linked_object_add_tail(&mqueue_list, (prelude_linked_object_t *) msg);
-        gl_lock_unlock(mqueue_mutex);
+        for ( i = 0; i < sizeof(receiver_list) / sizeof(*receiver_list); i++ ) {
+                prelude_list_for_each(&receiver_list[i], tmp) {
+                        receiver = prelude_list_entry(tmp, reverse_relay_receiver_t, list);
+                        if ( sender == receiver )
+                                continue;
 
-        return 0;
+                        ret = prelude_msg_clone(&clone, msg);
+                        if ( ret < 0 )
+                                break;
+
+                        bufpool_add_message(receiver->failover, clone);
+                        continue;
+                }
+        }
+
+        prelude_msg_destroy(msg);
+        return 1;
 }
 
 
@@ -253,25 +342,6 @@ static int get_issuer_analyzerid(idmef_message_t *idmef, uint64_t *analyzerid)
 
 
 
-static prelude_msg_t *mqueue_get_next(void)
-{
-        prelude_msg_t *m = NULL;
-
-        gl_lock_lock(mqueue_mutex);
-
-        if ( prelude_list_is_empty(&mqueue_list) )
-                goto out;
-
-        m = prelude_linked_object_get_object(mqueue_list.next);
-        prelude_linked_object_del((prelude_linked_object_t *) m);
-
-out:
-        gl_lock_unlock(mqueue_mutex);
-        return m;
-}
-
-
-
 void reverse_relay_send_receiver(idmef_message_t *idmef)
 {
         int ret;
@@ -317,34 +387,86 @@ void reverse_relay_send_receiver(idmef_message_t *idmef)
 
 void reverse_relay_send_prepared(void)
 {
-        int ret, i;
-        prelude_msg_t *m;
+
+        int i;
         prelude_list_t *tmp;
         reverse_relay_receiver_t *receiver;
+        sensor_fd_t *client;
 
-        while ( (m = mqueue_get_next()) ) {
+        for ( i = 0; i < sizeof(receiver_list) / sizeof(*receiver_list); i++ ) {
+                prelude_list_for_each(&receiver_list[i], tmp) {
+                        receiver = prelude_list_entry(tmp, reverse_relay_receiver_t, list);
+                        if ( ! receiver->failover || bufpool_get_message_count(receiver->failover) < 1 )
+                                continue;
 
-                for ( i = 0; i < sizeof(receiver_list) / sizeof(*receiver_list); i++ ) {
-                        prelude_list_for_each(&receiver_list[i], tmp) {
-                                receiver = prelude_list_entry(tmp, reverse_relay_receiver_t, list);
+                        client = find_client(receiver);
+                        if ( ! client || client->wmsg )
+                                continue;
 
-                                if ( prelude_msg_get_data(m) == receiver )
-                                        continue;
-
-                                ret = write_all_client(receiver, m, 0);
-                                if ( ! ret ) {
-                                        ret = prelude_failover_save_msg(receiver->failover, m);
-                                        if ( ret < 0 )
-                                                prelude_perror(ret, "could not save message to disk");
-                                }
-                        }
+                        reverse_relay_write_possible(receiver, (server_generic_client_t *) client);
                 }
-
-                prelude_msg_destroy(m);
         }
 }
 
 
+static void destroy_current_initiator(void)
+{
+        sensor_fd_t *client;
+        prelude_list_t *tmp;
+        prelude_connection_t *cnx;
+
+        prelude_list_for_each(prelude_connection_pool_get_connection_list(initiator), tmp) {
+                cnx = prelude_linked_object_get_object(tmp);
+
+                client = prelude_connection_get_data(cnx);
+                if ( client ) {
+                        client->cnx = NULL;
+                        client->fd = NULL;
+                        server_generic_remove_client(config.server[0], (server_generic_client_t *) client);
+                }
+        }
+
+        prelude_connection_pool_destroy(initiator);
+        initiator = NULL;
+}
+
+
+
+int reverse_relay_create_initiator(const char *arg)
+{
+        int ret;
+        prelude_client_profile_t *cp;
+
+        cp = prelude_client_get_profile(manager_client);
+
+        if ( initiator )
+                destroy_current_initiator();
+
+        ret = prelude_connection_pool_new(&initiator, cp, PRELUDE_CONNECTION_PERMISSION_IDMEF_READ);
+        if ( ret < 0 )
+                goto out;
+
+        prelude_connection_pool_set_flags(initiator, PRELUDE_CONNECTION_POOL_FLAGS_RECONNECT);
+        prelude_connection_pool_set_event_handler(initiator, PRELUDE_CONNECTION_POOL_EVENT_DEAD |
+                                                  PRELUDE_CONNECTION_POOL_EVENT_ALIVE, connection_event_cb);
+
+        ret = prelude_connection_pool_set_connection_string(initiator, arg);
+        if ( ret < 0 ) {
+                prelude_connection_pool_destroy(initiator);
+                goto out;
+        }
+
+        ret = prelude_connection_pool_init(initiator);
+        if ( ret < 0 ) {
+                prelude_connection_pool_destroy(initiator);
+                goto out;
+        }
+
+ out:
+        return ret;
+}
+
+
 
 int reverse_relay_init(void)
 {
